#include "glwidget.h"

#include <QOpenGLShaderProgram>
#include <QPainter>

GLWidget::GLWidget(QWidget *parent) :
    QOpenGLWidget(parent),
    m_program(0),
    m_vertexVBO(0),
    m_colorVBO(0){
}

GLWidget::~GLWidget() {
    delete m_program;
    glDeleteBuffers(1, &m_vertexVBO);
    glDeleteBuffers(1, &m_colorVBO);
}

void GLWidget::initializeGL() {
    // inizializziamo le funzioni opengl
    initializeOpenGLFunctions();

    // Creiamo l'oggetto QOpenGLShaderProgram
    m_program = new QOpenGLShaderProgram(this);

    // Compiliamo lo shader dei vertici
    if (!m_program->addShaderFromSourceFile(QOpenGLShader::Vertex, ":/vertex.glsl"))
        close();

    // Compiliamo il fragment shader
    if (!m_program->addShaderFromSourceFile(QOpenGLShader::Fragment, ":/fragment.glsl"))
        close();

    // Linkiamo gli shaders
    if (!m_program->link())
        close();

    // *** VBO dei vertici del triangolo ***
    // vettore contenete le coordinate dei vertici
    GLfloat vertexData[] = {
        GLfloat(0.0),  GLfloat(0.707), GLfloat(0.0),  GLfloat(1.0),
        GLfloat(-0.5), GLfloat(-0.50), GLfloat(0.0), GLfloat(1.0),
        GLfloat(0.5),  GLfloat(-0.50), GLfloat(0.0), GLfloat(1.0)
    };
    // genera il VBO
    glGenBuffers(1, &m_vertexVBO);
    // attiva il VBO
    glBindBuffer(GL_ARRAY_BUFFER, m_vertexVBO);
    // carica nella memoria della GPU i valori contenuti in vertexData
    glBufferData(GL_ARRAY_BUFFER, sizeof(vertexData), vertexData, GL_STATIC_DRAW);
    // disattiva il VBO
    glBindBuffer(GL_ARRAY_BUFFER, 0);

    // *** VBO dei colori ***
    // vettore contenete le coordinate dei colori
    GLfloat colorData[] = {
        GLfloat(1.0), GLfloat(0.0), GLfloat(0.0), GLfloat(1.0),
        GLfloat(0.0), GLfloat(1.0), GLfloat(0.0), GLfloat(1.0),
        GLfloat(0.0), GLfloat(0.0), GLfloat(1.0), GLfloat(1.0)
    };
    // genera il VBO
    glGenBuffers(1, &m_colorVBO);
    // attiva il VBO
    glBindBuffer(GL_ARRAY_BUFFER, m_colorVBO);
    // carica nella memoria della GPU i valori contenuti in colorData
    glBufferData(GL_ARRAY_BUFFER, sizeof(colorData), colorData, GL_STATIC_DRAW);
    // disattiva il VBO
    glBindBuffer(GL_ARRAY_BUFFER, 0);

    // Localizzaziamo gli attributi negli shader
    m_posAttr = m_program->attributeLocation("posAttr");
    m_colAttr = m_program->attributeLocation("colAttr");
    m_matrixUniform = m_program->uniformLocation("matrix");

    // impostiamo la prospettiva
    m_matrix.perspective(60, 4.0/3.0, 0.1, 100.0);
    m_matrix.translate(0, 0, -2);
}

void GLWidget::resizeGL(int w, int h) {
    glViewport(0, 0, w, h);
}

void GLWidget::paintGL() {
    // Usiamo QPainter perché vogliamo usarlo per integrare il disegno opengl
    QPainter painter;

    painter.begin(this);

    painter.beginNativePainting();

    // **** Disegni opengl ****

    // Rendiamo corrente il contesto - in questo caso non è
    // necessario, ma è una buona abitudine
    makeCurrent();

    // Puliamo la la vista
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

    // Linkiamo gli shader
    m_program->bind();

    // Carichiamo la matrice modello/vista
    m_program->setUniformValue(m_matrixUniform, m_matrix);

    // Attiviamo il VBO dei vertici
    glBindBuffer(GL_ARRAY_BUFFER, m_vertexVBO); // attiva il VBO
    glEnableVertexAttribArray( m_posAttr );
    glVertexAttribPointer( m_posAttr,
                           4,                   // numero di elementi per vertice nel nostro caso 4 (x,y,z,w)
                           GL_FLOAT,            // tipo di elemento
                           GL_FALSE,
                           0,
                           0 );

    // Attiviamo il VBO dei colori
    glBindBuffer(GL_ARRAY_BUFFER, m_colorVBO);  // attiva il VBO
    glEnableVertexAttribArray( m_colAttr );
    glVertexAttribPointer( m_colAttr,
                           4,                   // numero di elementi per vertice nel nostro caso 4 (x,y,z,w)
                           GL_FLOAT,            // tipo di elemento
                           GL_FALSE,
                           0,
                           0 );

    // Disegniamo il triangolo
    glDrawArrays(GL_TRIANGLES, 0, 3);

    // Rilasciamo gli shader
    m_program->release();

    painter.endNativePainting();

    painter.end();
}
